@@ -1,17 +1,11 @@
 rxPermissive({
     context("lhs for different compartments make sense")
 
-    et <- eventTable() %>% add.dosing(dose=3, nbr.doses=6, dosing.interval=8) %>%
-        add.sampling(seq(0, 48, length.out=200))
+    et <- et() %>% et(amt=3, addl=5, ii=8,cmt="depot") %>%
+        et(seq(0,48,length.out=200),cmt="depot") %>%
+        et(seq(0,48,length.out=200),cmt="centr") %>%
+        et(seq(0,48,length.out=200),cmt="peri")
 
-    df <- et$get.sampling()
-
-    df <- rbind(data.frame(et$get.dosing(), CMT=1),
-                data.frame(df, CMT = 1),
-                data.frame(df, CMT = 2),
-                data.frame(df, CMT = 3))
-
-    df <- df[order(df$time, df$CMT), ]
 
     ode.2c.ka <- RxODE({
         C2 = centr/V;
@@ -31,13 +25,28 @@ rxPermissive({
         }
     })
 
-    test_that("multi-compartment solves", {
-        s <- rxSolve(ode.2c.ka, params=c(V=40, CL=18, V2=297, Q=10, KA= 0.3), df)
+    tmp <- etTrans(et, ode.2c.ka)
 
+    test_that("multi-compartment solves", {
+        s <- rxSolve(ode.2c.ka, params=c(V=40, CL=18, V2=297, Q=10, KA= 0.3), et)
         expect_equal(sort(unique(s$C4)), c(1, 2, 3))
         expect_equal(s[s$C4 == 1, "prd"], s[s$C4 == 1, "depot"])
         expect_equal(s[s$C4 == 2, "prd"], s[s$C4 == 2, "centr"])
         expect_equal(s[s$C4 == 3, "prd"], s[s$C4 == 3, "peri"])
-
     })
+
+    ## Now change cmt to an un-ordered factor.
+    et <- as.data.frame(et);
+    et$cmt <- factor(et$cmt, c("peri", "depot", "centr"), c("peri", "depot", "centr"))
+
+    tmp <- etTrans(et, ode.2c.ka);
+
+    test_that("multi-compartment solves", {
+        s <- rxSolve(ode.2c.ka, params=c(V=40, CL=18, V2=297, Q=10, KA= 0.3), et)
+        expect_equal(sort(unique(s$C4)), c(1, 2, 3))
+        expect_equal(s[s$C4 == 1, "prd"], s[s$C4 == 1, "depot"])
+        expect_equal(s[s$C4 == 2, "prd"], s[s$C4 == 2, "centr"])
+        expect_equal(s[s$C4 == 3, "prd"], s[s$C4 == 3, "peri"])
+    })
+
 })
